/*
 *    GS2MacroResolver.java
 *    Copyright (C) 2005 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.util;

import java.util.ArrayList;

public class GS2MacroResolver 
    extends MacroResolver 
{

    protected GDBMWrapper gdbm_src = null;
    
    // need to make it not add macros if they are already present
    public GS2MacroResolver(GDBMWrapper gdbm) {
	super();
	gdbm_src = gdbm;
    }

    
    public String resolve(String text, String lang, String scope, 
			  String doc_oid) {
	if (text == null || text.equals("")) return text;
	if (scope.equals(SCOPE_TEXT) && text_macros.size()==0) return text;
	if (scope.equals(SCOPE_META) && metadata_macros.size() ==0) return text;
	DBInfo node_info = null;
	DBInfo root_info = null;
	boolean new_lang = false;
	if (this.lang == null || !this.lang.equals(lang) ) {
	    new_lang = true;
	    this.lang = lang;
	} 

	ArrayList macros;
	if (scope.equals(SCOPE_TEXT)) {
	    macros = text_macros;
	} else {
	    macros = metadata_macros;
	}
	for (int i=0; i<macros.size(); i++) {
	    String new_text = null;
	    Macro m = (Macro)macros.get(i);
	    switch (m.type) {
	    case TYPE_DICT:
		if (m.text==null || new_lang) {
		    Dictionary dict = new Dictionary(m.bundle, lang);
		    m.text = dict.get(m.key, null);
		}
		// we assume that dictionary entries will contain no macros
		// otherwise we can't cache the answer because it might be 
		// document specific
		text = text.replaceAll(m.macro, m.text);
		break;
	    case TYPE_TEXT:
		// make sure we resolve any macros in the text
		// the (?s) treats the string as a single line, cos . 
		// doesn't necessarily match line breaks
		if (text.matches("(?s).*"+m.macro+".*")) {
		    if (m.resolve) {
			new_text = this.resolve(m.text, lang, scope, doc_oid);
		    } else {
			new_text = m.text;
		    }
		    text = text.replaceAll(m.macro, new_text);
		}
		break;
	    case TYPE_META:
		if (text.matches(".*"+m.macro+".*")) {
		    if (node_info == null) {
			node_info = gdbm_src.getInfo(doc_oid);
			if (node_info == null) {
			    break;
			}
		    }
		    new_text = node_info.getInfo(m.text);
		    if (new_text == null || new_text.equals("")) {
			// try the root node
			if (root_info == null && !OID.isTop(doc_oid)) {
			    root_info = gdbm_src.getInfo(OID.getTop(doc_oid));
			}
			if (root_info == null) break;
			new_text = root_info.getInfo(m.text);
		    }
		    if (new_text != null) {
			if (m.resolve) {
			    new_text = this.resolve(new_text, lang, scope, doc_oid);
			}
			text = text.replaceAll(m.macro, new_text);
		    }
		    
		}
		
		break;
	    } // switch
	   
	}
	return text;
	
    }

}
